import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';

/** Extra properties that are not present in the default PaletteOptions of mui. For autocompletion */
export interface ExtraColorsForMui5 {
  /** Colors that can be used for data visualisation, e.g. nodes, edges */
  dataPointColors: string[];

  queryBuilder: {
    text: string;
    entity: {
      background: string;
      lighterbg?: string;
    };
    relation: {
      background: string;
      lighterbg?: string;
    };
    attribute: {
      background: string;
      lighterbg?: string;
    };
  };
}

/** Our custom color palette config. With the palette options from MUI that we are going to use. */
export interface ColorPaletteConfig {
  lightPalette: {
    // Custom colors
    custom: ExtraColorsForMui5;
    // MUI colors
    primary: {
      light?: string;
      main: string;
      dark?: string;
    };
    secondary: {
      light?: string;
      main: string;
      dark?: string;
    };
  };
  darkPalette: {
    custom: ExtraColorsForMui5;
    primary: {
      light?: string;
      main: string;
      dark?: string;
    };
    secondary: {
      light?: string;
      main: string;
      dark?: string;
    };
  };
  darkMode: boolean;
}

// This looks very much like the mui Palette interface,
// But we don't reference that type directly here to stay decoupled
export const initialState: ColorPaletteConfig = {
  lightPalette: {
    custom: {
      dataPointColors: ['#ff0000', '#00ff00', '#0000ff'],
      queryBuilder: {
        text: 'black',
        entity: {
          background: '#FC4F4F',
        },
        relation: {
          background: '#FF9F45',
        },
        attribute: {
          background: '#C7C7C7',
        },
      },
    },
    // If light and dark are not set, these will be calculated using main.
    // light/dark have nothing with darkmode, these are just a light and dark variation
    primary: {
      // light: '#42a5f5',
      main: '#1976d2',
      // dark: '#1565c0',
    },
    secondary: {
      // light: '#ba68c8',
      main: '#9c27b0',
      // dark: '#7b1fa2',
    },
  },
  darkPalette: {
    custom: {
      dataPointColors: ['#ff0000', '#00ff00', '#0000ff'],
      queryBuilder: {
        text: 'black',
        entity: {
          background: '#FC4F4F',
        },
        relation: {
          background: '#FF9F45',
        },
        attribute: {
          background: '#C7C7C7',
        },
      },
    },
    primary: {
      main: '#e3f3fd',
    },
    secondary: {
      main: '#f3e5f5',
    },
  },
  darkMode: false,
};

export const colorPaletteConfigSlice = createSlice({
  name: 'colorPaletteConfig',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    changePrimary: (
      state,
      action: PayloadAction<{
        main?: string;
        light?: string;
        dark?: string;
        darkMode?: 'light' | 'dark';
      }>
    ) => {
      const { main, light, dark, darkMode } = action.payload;
      let palette = state.lightPalette;
      if (darkMode == 'dark') palette = state.darkPalette;

      if (main) palette.primary.main = main;
      if (light) palette.primary.light = light;
      if (dark) palette.primary.dark = dark;
    },
    changeDataPointColors: (state, action: PayloadAction<string[]>) => {
      state.lightPalette.custom.dataPointColors = action.payload;
    },
    toggleDarkMode: (state) => {
      state.darkMode = !state.darkMode;
    },
  },
});

export const { changePrimary, changeDataPointColors, toggleDarkMode } =
  colorPaletteConfigSlice.actions;

// Select the schema and convert it to a graphology object
export const selectColorPaletteConfig = (state: RootState) =>
  state.colorPaletteConfig;

export default colorPaletteConfigSlice.reducer;
